@@ -1,5 +1,5 @@
-use cargo_test_support::prelude::*;
-use cargo_test_support::{compare::assert_ui, current_dir, file, Project};
+use crate::prelude::*;
+use cargo_test_support::{Project, compare::assert_ui, current_dir, file};
 
 use super::init_registry_without_token;
 
