@@ -25,9 +25,9 @@ declare_tool_lint! {
 declare_lint_pass!(ProduceIce => [PRODUCE_ICE]);
 
 impl EarlyLintPass for ProduceIce {
-    fn check_fn(&mut self, ctx: &EarlyContext<'_>, fn_kind: FnKind<'_>, span: Span, _: NodeId) {
+    fn check_fn(&mut self, cx: &EarlyContext<'_>, fn_kind: FnKind<'_>, span: Span, _: NodeId) {
         if is_trigger_fn(fn_kind) {
-            ctx.sess()
+            cx.sess()
                 .dcx()
                 .span_delayed_bug(span, "Would you like some help with that?");
         }
