@@ -1,7 +1,6 @@
-// run-rustfix
-// aux-build:proc_macros.rs
+//@aux-build:proc_macros.rs
 #![warn(clippy::toplevel_ref_arg)]
-#![allow(clippy::uninlined_format_args, unused)]
+#![allow(clippy::uninlined_format_args, unused, clippy::useless_vec)]
 
 extern crate proc_macros;
 use proc_macros::{external, inline_macros};
@@ -13,17 +12,22 @@ fn main() {
     y(1u8);
 
     let ref _x = 1;
+    //~^ toplevel_ref_arg
 
     let ref _y: (&_, u8) = (&1, 2);
+    //~^ toplevel_ref_arg
 
     let ref _z = 1 + 2;
+    //~^ toplevel_ref_arg
 
     let ref mut _z = 1 + 2;
+    //~^ toplevel_ref_arg
 
     let (ref x, _) = (1, 2); // ok, not top level
     println!("The answer is {}.", x);
 
     let ref _x = vec![1, 2, 3];
+    //~^ toplevel_ref_arg
 
     // Make sure that allowing the lint works
     #[allow(clippy::toplevel_ref_arg)]
@@ -34,7 +38,10 @@ fn main() {
 
     // lint in macro
     inline!(let ref _y = 42;);
+    //~^ toplevel_ref_arg
 
     // do not lint in external macro
     external!(let ref _y = 42;);
+
+    fn f(#[allow(clippy::toplevel_ref_arg)] ref x: i32) {}
 }
