@@ -1,9 +1,11 @@
-#![feature(control_flow_enum)]
+// skip-filecheck
 #![feature(try_trait_v2)]
 
+//@ compile-flags: -Zunsound-mir-opts
+
 use std::ops::ControlFlow;
 
-// EMIT_MIR separate_const_switch.too_complex.SeparateConstSwitch.diff
+// EMIT_MIR separate_const_switch.too_complex.JumpThreading.diff
 fn too_complex(x: Result<i32, usize>) -> Option<i32> {
     // The pass should break the outer match into
     // two blocks that only have one parent each.
@@ -20,7 +22,7 @@ fn too_complex(x: Result<i32, usize>) -> Option<i32> {
     }
 }
 
-// EMIT_MIR separate_const_switch.identity.SeparateConstSwitch.diff
+// EMIT_MIR separate_const_switch.identity.JumpThreading.diff
 fn identity(x: Result<i32, i32>) -> Result<i32, i32> {
     Ok(x?)
 }
