@@ -2,11 +2,9 @@
 
 fn main() {
     // intrinsics are guarded by contracts_internals feature gate.
-    core::intrinsics::contract_checks();
-    //~^ ERROR use of unstable library feature `contracts_internals`
     core::intrinsics::contract_check_requires(|| true);
     //~^ ERROR use of unstable library feature `contracts_internals`
-    core::intrinsics::contract_check_ensures( |_|true, &1);
+    core::intrinsics::contract_check_ensures(Some(|_: &&u32| true), &1);
     //~^ ERROR use of unstable library feature `contracts_internals`
 
     core::contracts::build_check_ensures(|_: &()| true);
