@@ -3,7 +3,7 @@
 // valid.
 
 
-#![allow(dead_code)]
+#![allow(dead_code, unused)]
 
 trait Test {
     fn foo(&self) { }
