@@ -1,6 +1,6 @@
-// run-rustfix
+//@ edition: 2015
+//@ run-rustfix
 
-#![feature(rust_2018_preview)]
 #![deny(absolute_paths_not_starting_with_crate)]
 #![allow(unused_imports)]
 #![allow(dead_code)]
