@@ -1,17 +1,13 @@
 //! Ensure ABI-incompatible features cannot be enabled via `#[target_feature]`.
 //@ compile-flags: --target=riscv64gc-unknown-linux-gnu --crate-type=lib
 //@ needs-llvm-components: riscv
-#![feature(no_core, lang_items, riscv_target_feature)]
+//@ ignore-backends: gcc
+//@ add-core-stubs
+#![feature(no_core, riscv_target_feature)]
 #![no_core]
 
-#[lang = "pointee_sized"]
-pub trait PointeeSized {}
-
-#[lang = "meta_sized"]
-pub trait MetaSized: PointeeSized {}
-
-#[lang = "sized"]
-pub trait Sized {}
+extern crate minicore;
+use minicore::*;
 
 #[target_feature(enable = "zdinx")]
 //~^ERROR: cannot be enabled with
