@@ -1,17 +1,13 @@
 //! Ensure "forbidden" target features cannot be enabled via `#[target_feature]`.
 //@ compile-flags: --target=riscv32e-unknown-none-elf --crate-type=lib
 //@ needs-llvm-components: riscv
-#![feature(no_core, lang_items)]
+//@ ignore-backends: gcc
+//@ add-core-stubs
+#![feature(no_core)]
 #![no_core]
 
-#[lang = "pointee_sized"]
-pub trait PointeeSized {}
-
-#[lang = "meta_sized"]
-pub trait MetaSized: PointeeSized {}
-
-#[lang = "sized"]
-pub trait Sized: MetaSized {}
+extern crate minicore;
+use minicore::*;
 
 #[target_feature(enable = "forced-atomics")]
 //~^ERROR: cannot be enabled with
